@@ -1,9 +1,11 @@
 /*
- * regexp.h : describes the basic API for libxml regular expressions handling
+ * Summary: regular expressions handling
+ * Description: basic API for libxml regular expressions handling used
+ *              for XML Schemas and validation.
  *
- * See Copyright for the status of this software.
+ * Copy: See Copyright for the status of this software.
  *
- * Daniel Veillard <veillard@redhat.com>
+ * Author: Daniel Veillard
  */
 
 #ifndef __XML_REGEXP_H__
@@ -45,13 +47,17 @@ extern "C" {
 /*
  * The POSIX like API
  */
-xmlRegexpPtr		xmlRegexpCompile(const xmlChar *regexp);
-void			xmlRegFreeRegexp(xmlRegexpPtr regexp);
-int			xmlRegexpExec	(xmlRegexpPtr comp,
+XMLPUBFUN xmlRegexpPtr XMLCALL
+		    xmlRegexpCompile	(const xmlChar *regexp);
+XMLPUBFUN void XMLCALL			 xmlRegFreeRegexp(xmlRegexpPtr regexp);
+XMLPUBFUN int XMLCALL			
+		    xmlRegexpExec	(xmlRegexpPtr comp,
 					 const xmlChar *value);
-void			xmlRegexpPrint	(FILE *output,
+XMLPUBFUN void XMLCALL			
+    		    xmlRegexpPrint	(FILE *output,
 					 xmlRegexpPtr regexp);
-int			xmlRegexpIsDeterminist(xmlRegexpPtr comp);
+XMLPUBFUN int XMLCALL			
+		    xmlRegexpIsDeterminist(xmlRegexpPtr comp);
 
 /*
  * Callback function when doing a transition in the automata
@@ -64,17 +70,21 @@ typedef void (*xmlRegExecCallbacks) (xmlRegExecCtxtPtr exec,
 /*
  * The progressive API
  */
-xmlRegExecCtxtPtr	xmlRegNewExecCtxt	(xmlRegexpPtr comp,
-						 xmlRegExecCallbacks callback,
-						 void *data);
-void			xmlRegFreeExecCtxt	(xmlRegExecCtxtPtr exec);
-int			xmlRegExecPushString	(xmlRegExecCtxtPtr exec,
-						 const xmlChar *value,
-						 void *data);
-int			xmlRegExecPushString2	(xmlRegExecCtxtPtr exec,
-						 const xmlChar *value,
-						 const xmlChar *value2,
-						 void *data);
+XMLPUBFUN xmlRegExecCtxtPtr XMLCALL	
+    		    xmlRegNewExecCtxt	(xmlRegexpPtr comp,
+					 xmlRegExecCallbacks callback,
+					 void *data);
+XMLPUBFUN void XMLCALL			
+		    xmlRegFreeExecCtxt	(xmlRegExecCtxtPtr exec);
+XMLPUBFUN int XMLCALL			
+    		    xmlRegExecPushString(xmlRegExecCtxtPtr exec,
+					 const xmlChar *value,
+					 void *data);
+XMLPUBFUN int XMLCALL			
+		    xmlRegExecPushString2(xmlRegExecCtxtPtr exec,
+					 const xmlChar *value,
+					 const xmlChar *value2,
+					 void *data);
 
 #ifdef __cplusplus
 }
