@@ -29,6 +29,7 @@
 #include <ctype.h>
 #endif
 
+/* #define DEBUG_MEMORY */
 
 /**
  * MEM_LIST:
@@ -42,13 +43,15 @@
 #endif
 #endif
 
+#include <libxml/globals.h>	/* must come before xmlmemory.h */
 #include <libxml/xmlmemory.h>
-#include <libxml/globals.h>
 #include <libxml/xmlerror.h>
+#include <libxml/threads.h>
 
 static int xmlMemInitialized = 0;
 static unsigned long  debugMemSize = 0;
 static unsigned long  debugMaxMemSize = 0;
+static xmlMutexPtr xmlMemMutex = NULL;
 
 void xmlMallocBreakpoint(void);
 
@@ -58,7 +61,7 @@ void xmlMallocBreakpoint(void);
  *									*
  ************************************************************************/
 
-
+#if !defined(LIBXML_THREAD_ENABLED) && !defined(LIBXML_THREAD_ALLOC_ENABLED)
 #ifdef xmlMalloc
 #undef xmlMalloc
 #endif
@@ -68,7 +71,7 @@ void xmlMallocBreakpoint(void);
 #ifdef xmlMemStrdup
 #undef xmlMemStrdup
 #endif
-
+#endif
 
 /*
  * Each of the blocks allocated begin with a header containing informations
@@ -110,8 +113,8 @@ typedef struct memnod {
 #define HDR_2_CLIENT(a)    ((void *) (((char *) (a)) + RESERVE_SIZE))
 
 
-static int block=0;
-static int xmlMemStopAtBlock = 0;
+static unsigned int block=0;
+static unsigned int xmlMemStopAtBlock = 0;
 static void *xmlMemTraceBlockAt = NULL;
 #ifdef MEM_LIST
 static MEMHDR *memlist = NULL;
@@ -176,23 +179,25 @@ xmlMallocLoc(size_t size, const char * file, int line)
 	return(NULL);
     }   
     p->mh_tag = MEMTAG;
-    p->mh_number = ++block;
     p->mh_size = size;
     p->mh_type = MALLOC_TYPE;
     p->mh_file = file;
     p->mh_line = line;
+    xmlMutexLock(xmlMemMutex);
+    p->mh_number = ++block;
     debugMemSize += size;
     if (debugMemSize > debugMaxMemSize) debugMaxMemSize = debugMemSize;
 #ifdef MEM_LIST
     debugmem_list_add(p);
 #endif
-
+    xmlMutexUnlock(xmlMemMutex);
+    
 #ifdef DEBUG_MEMORY
     xmlGenericError(xmlGenericErrorContext,
 	    "Malloc(%d) Ok\n",size);
 #endif
     
-    if (xmlMemStopAtBlock == block) xmlMallocBreakpoint();
+    if (xmlMemStopAtBlock == p->mh_number) xmlMallocBreakpoint();
 
     ret = HDR_2_CLIENT(p);
 
@@ -241,23 +246,25 @@ xmlMallocAtomicLoc(size_t size, const char * file, int line)
 	return(NULL);
     }   
     p->mh_tag = MEMTAG;
-    p->mh_number = ++block;
     p->mh_size = size;
     p->mh_type = MALLOC_ATOMIC_TYPE;
     p->mh_file = file;
     p->mh_line = line;
+    xmlMutexLock(xmlMemMutex);
+    p->mh_number = ++block;
     debugMemSize += size;
     if (debugMemSize > debugMaxMemSize) debugMaxMemSize = debugMemSize;
 #ifdef MEM_LIST
     debugmem_list_add(p);
 #endif
+    xmlMutexUnlock(xmlMemMutex);
 
 #ifdef DEBUG_MEMORY
     xmlGenericError(xmlGenericErrorContext,
 	    "Malloc(%d) Ok\n",size);
 #endif
     
-    if (xmlMemStopAtBlock == block) xmlMallocBreakpoint();
+    if (xmlMemStopAtBlock == p->mh_number) xmlMallocBreakpoint();
 
     ret = HDR_2_CLIENT(p);
 
@@ -303,10 +310,14 @@ xmlReallocLoc(void *ptr,size_t size, const char * file, int line)
 {
     MEMHDR *p;
     unsigned long number;
+#ifdef DEBUG_MEMORY
+    size_t oldsize;
+#endif
+
+    if (ptr == NULL)
+        return(xmlMallocLoc(size, file, line));
 
     if (!xmlMemInitialized) xmlInitMemory();
-    if (ptr == NULL)
-        return(NULL);
     TEST_POINT
 
     p = CLIENT_2_HDR(ptr);
@@ -316,11 +327,16 @@ xmlReallocLoc(void *ptr,size_t size, const char * file, int line)
 	 goto error;
     }
     p->mh_tag = ~MEMTAG;
+    xmlMutexLock(xmlMemMutex);
     debugMemSize -= p->mh_size;
+#ifdef DEBUG_MEMORY
+    oldsize = p->mh_size;
+#endif
 #ifdef MEM_LIST
     debugmem_list_delete(p);
 #endif
-
+    xmlMutexUnlock(xmlMemMutex);
+    
     p = (MEMHDR *) realloc(p,RESERVE_SIZE+size);
     if (!p) {
 	 goto error;
@@ -337,14 +353,20 @@ xmlReallocLoc(void *ptr,size_t size, const char * file, int line)
     p->mh_size = size;
     p->mh_file = file;
     p->mh_line = line;
+    xmlMutexLock(xmlMemMutex);
     debugMemSize += size;
     if (debugMemSize > debugMaxMemSize) debugMaxMemSize = debugMemSize;
 #ifdef MEM_LIST
     debugmem_list_add(p);
 #endif
+    xmlMutexUnlock(xmlMemMutex);
 
     TEST_POINT
 
+#ifdef DEBUG_MEMORY
+    xmlGenericError(xmlGenericErrorContext,
+	    "Realloced(%d to %d) Ok\n", oldsize, size);
+#endif
     return(HDR_2_CLIENT(p));
     
 error:    
@@ -377,6 +399,9 @@ xmlMemFree(void *ptr)
 {
     MEMHDR *p;
     char *target;
+#ifdef DEBUG_MEMORY
+    size_t size;
+#endif
 
     if (ptr == (void *) -1) {
 	xmlGenericError(xmlGenericErrorContext,
@@ -400,16 +425,26 @@ xmlMemFree(void *ptr)
         goto error;
     }
     p->mh_tag = ~MEMTAG;
-    debugMemSize -= p->mh_size;
     memset(target, -1, p->mh_size);
-
+    xmlMutexLock(xmlMemMutex);
+    debugMemSize -= p->mh_size;
+#ifdef DEBUG_MEMORY
+    size = p->mh_size;
+#endif
 #ifdef MEM_LIST
     debugmem_list_delete(p);
 #endif
+    xmlMutexUnlock(xmlMemMutex);
+
     free(p);
 
     TEST_POINT
 
+#ifdef DEBUG_MEMORY
+    xmlGenericError(xmlGenericErrorContext,
+	    "Freed(%d) Ok\n", size);
+#endif
+    
     return;
     
 error:    
@@ -445,19 +480,22 @@ xmlMemStrdupLoc(const char *str, const char *file, int line)
       goto error;
     }
     p->mh_tag = MEMTAG;
-    p->mh_number = ++block;
     p->mh_size = size;
     p->mh_type = STRDUP_TYPE;
     p->mh_file = file;
     p->mh_line = line;
+    xmlMutexLock(xmlMemMutex);
+    p->mh_number = ++block;
     debugMemSize += size;
     if (debugMemSize > debugMaxMemSize) debugMaxMemSize = debugMemSize;
 #ifdef MEM_LIST
     debugmem_list_add(p);
 #endif
+    xmlMutexUnlock(xmlMemMutex);
+    
     s = (char *) HDR_2_CLIENT(p);
     
-    if (xmlMemStopAtBlock == block) xmlMallocBreakpoint();
+    if (xmlMemStopAtBlock == p->mh_number) xmlMallocBreakpoint();
 
     if (s != NULL)
       strcpy(s,str);
@@ -594,6 +632,7 @@ xmlMemDisplay(FILE *fp)
             debugMemSize, debugMaxMemSize);
     fprintf(fp,"BLOCK  NUMBER   SIZE  TYPE\n");
     idx = 0;
+    xmlMutexLock(xmlMemMutex);
     p = memlist;
     while (p) {
 	  fprintf(fp,"%-5u  %6lu %6lu ",idx++,p->mh_number,
@@ -601,12 +640,15 @@ xmlMemDisplay(FILE *fp)
         switch (p->mh_type) {
            case STRDUP_TYPE:fprintf(fp,"strdup()  in ");break;
            case MALLOC_TYPE:fprintf(fp,"malloc()  in ");break;
-          case REALLOC_TYPE:fprintf(fp,"realloc() in ");break;
+           case REALLOC_TYPE:fprintf(fp,"realloc() in ");break;
            case MALLOC_ATOMIC_TYPE:fprintf(fp,"atomicmalloc()  in ");break;
-          case REALLOC_ATOMIC_TYPE:fprintf(fp,"atomicrealloc() in ");break;
-                    default:fprintf(fp,"   ???    in ");break;
+           case REALLOC_ATOMIC_TYPE:fprintf(fp,"atomicrealloc() in ");break;
+           default:
+	        fprintf(fp,"Unknown memory block, may be corrupted");
+		xmlMutexUnlock(xmlMemMutex);
+		return;
         }
-	  if (p->mh_file != NULL) fprintf(fp,"%s(%d)", p->mh_file, p->mh_line);
+	if (p->mh_file != NULL) fprintf(fp,"%s(%u)", p->mh_file, p->mh_line);
         if (p->mh_tag != MEMTAG)
 	      fprintf(fp,"  INVALID");
         nb++;
@@ -618,6 +660,7 @@ xmlMemDisplay(FILE *fp)
         fprintf(fp,"\n");
         p = p->mh_next;
     }
+    xmlMutexUnlock(xmlMemMutex);
 #else
     fprintf(fp,"Memory list not compiled (MEM_LIST not defined !)\n");
 #endif
@@ -692,6 +735,7 @@ xmlMemShow(FILE *fp, int nr ATTRIBUTE_UNUSED)
 	fprintf(fp,"      MEMORY ALLOCATED : %lu, MAX was %lu\n",
 		debugMemSize, debugMaxMemSize);
 #ifdef MEM_LIST
+    xmlMutexLock(xmlMemMutex);
     if (nr > 0) {
 	fprintf(fp,"NUMBER   SIZE  TYPE   WHERE\n");
 	p = memlist;
@@ -706,7 +750,7 @@ xmlMemShow(FILE *fp, int nr ATTRIBUTE_UNUSED)
 		default:fprintf(fp,"   ???    in ");break;
 	    }
 	    if (p->mh_file != NULL)
-	        fprintf(fp,"%s(%d)", p->mh_file, p->mh_line);
+	        fprintf(fp,"%s(%u)", p->mh_file, p->mh_line);
 	    if (p->mh_tag != MEMTAG)
 		fprintf(fp,"  INVALID");
 	    xmlMemContentShow(fp, p);
@@ -715,6 +759,7 @@ xmlMemShow(FILE *fp, int nr ATTRIBUTE_UNUSED)
 	    p = p->mh_next;
 	}
     }
+    xmlMutexUnlock(xmlMemMutex);
 #endif /* MEM_LIST */    
 }
 
@@ -750,8 +795,6 @@ xmlMemoryDump(void)
  *								*
  ****************************************************************/
 
-static int xmlInitMemoryDone = 0;
-
 /**
  * xmlInitMemory:
  *
@@ -765,13 +808,22 @@ xmlInitMemory(void)
 #ifdef HAVE_STDLIB_H
      char *breakpoint;
 #endif     
-
-     if (xmlInitMemoryDone) return(-1);
+#ifdef DEBUG_MEMORY
+     xmlGenericError(xmlGenericErrorContext,
+	     "xmlInitMemory()\n");
+#endif     
+    /*
+     This is really not good code (see Bug 130419).  Suggestions for
+     improvement will be welcome!
+    */
+     if (xmlMemInitialized) return(-1);
+     xmlMemInitialized = 1;
+     xmlMemMutex = xmlNewMutex();
 
 #ifdef HAVE_STDLIB_H
      breakpoint = getenv("XML_MEM_BREAKPOINT");
      if (breakpoint != NULL) {
-         sscanf(breakpoint, "%d", &xmlMemStopAtBlock);
+         sscanf(breakpoint, "%ud", &xmlMemStopAtBlock);
      }
 #endif     
 #ifdef HAVE_STDLIB_H
@@ -785,12 +837,32 @@ xmlInitMemory(void)
      xmlGenericError(xmlGenericErrorContext,
 	     "xmlInitMemory() Ok\n");
 #endif     
-     xmlMemInitialized = 1;
-     xmlInitMemoryDone = 1;
-
      return(0);
 }
 
+/**
+ * xmlCleanupMemory:
+ *
+ * Free up all the memory associated with memorys
+ */
+void
+xmlCleanupMemory(void) {
+#ifdef DEBUG_MEMORY
+     xmlGenericError(xmlGenericErrorContext,
+	     "xmlCleanupMemory()\n");
+#endif     
+    if (xmlMemInitialized == 0)
+        return;
+
+    xmlFreeMutex(xmlMemMutex);
+    xmlMemMutex = NULL;
+    xmlMemInitialized = 0;
+#ifdef DEBUG_MEMORY
+     xmlGenericError(xmlGenericErrorContext,
+	     "xmlCleanupMemory() Ok\n");
+#endif     
+}
+
 /**
  * xmlMemSetup:
  * @freeFunc: the free() function to use
@@ -809,6 +881,10 @@ xmlInitMemory(void)
 int
 xmlMemSetup(xmlFreeFunc freeFunc, xmlMallocFunc mallocFunc,
             xmlReallocFunc reallocFunc, xmlStrdupFunc strdupFunc) {
+#ifdef DEBUG_MEMORY
+     xmlGenericError(xmlGenericErrorContext,
+	     "xmlMemSetup()\n");
+#endif     
     if (freeFunc == NULL)
 	return(-1);
     if (mallocFunc == NULL)
@@ -822,6 +898,10 @@ xmlMemSetup(xmlFreeFunc freeFunc, xmlMallocFunc mallocFunc,
     xmlMallocAtomic = mallocFunc;
     xmlRealloc = reallocFunc;
     xmlMemStrdup = strdupFunc;
+#ifdef DEBUG_MEMORY
+     xmlGenericError(xmlGenericErrorContext,
+	     "xmlMemSetup() Ok\n");
+#endif     
     return(0);
 }
 
@@ -868,6 +948,10 @@ int
 xmlGcMemSetup(xmlFreeFunc freeFunc, xmlMallocFunc mallocFunc,
               xmlMallocFunc mallocAtomicFunc, xmlReallocFunc reallocFunc,
 	      xmlStrdupFunc strdupFunc) {
+#ifdef DEBUG_MEMORY
+     xmlGenericError(xmlGenericErrorContext,
+	     "xmlGcMemSetup()\n");
+#endif     
     if (freeFunc == NULL)
 	return(-1);
     if (mallocFunc == NULL)
@@ -883,6 +967,10 @@ xmlGcMemSetup(xmlFreeFunc freeFunc, xmlMallocFunc mallocFunc,
     xmlMallocAtomic = mallocAtomicFunc;
     xmlRealloc = reallocFunc;
     xmlMemStrdup = strdupFunc;
+#ifdef DEBUG_MEMORY
+     xmlGenericError(xmlGenericErrorContext,
+	     "xmlGcMemSetup() Ok\n");
+#endif     
     return(0);
 }
 
